package stx.libjava.tools.compiler.ecj;

import java.io.File;
import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;

public class CompilationUnit implements ICompilationUnit {
	/**
	 * A full Java (dotted) type name, i.e., java.lang.String.
	 * If compilation unit contains more than one type, this is the name
	 * of the main type. 
	 */
	protected String name;
	
	/**
	 * Source of the class
	 */
	protected String source;

	/**
	 * Creates a new compilation unit. 
	 * 
	 * @param name A fully qualified Java name of the main type contained in the source. 
	 * @param source Source code
	 */
	public CompilationUnit(String name,String source) {
		this.name = name;
		this.source = source;
	}
	
	@Override
	public char[] getFileName() {		
		return (name.substring(name.lastIndexOf('.') + 1) + ".java").toCharArray();
	}

	@Override
	public char[] getContents() {
		return source.toCharArray();
	}

	@Override
	public char[] getMainTypeName() {
		return name.substring(name.lastIndexOf('.') + 1).toCharArray();		
	}

	@Override
	public char[][] getPackageName() {
		char[] nameC = name.toCharArray();
		int ncomponents = 0;
		int i;
		for (i = 0; i < nameC.length; i++) {
			if (nameC[i] == '.') ncomponents++; 
		}
		char[][] packagename = new char[ncomponents][];
		int j = 0;
		int ci = 0;
		for (i = 0; i < nameC.length; i++) {
			if (nameC[i] == '.') {
				char[] component = new char[i - j];
				packagename[ci++] = component;
				System.arraycopy(nameC, j, component, 0, i-j);
				j = i+1;
			}
		}
		return packagename;
	}

	@Override
	public boolean ignoreOptionalProblems() {
		// TODO Auto-generated method stub
		return false;
	}
}
